/*
Trajectorize

Stumpuff functions

This file contains headers for C(z) and S(z) functions
used to solve the Universal Kepler Equation.
*/

#ifndef STUMPUFF_FUNCTIONS_H
#define STUMPUFF_FUNCTIONS_H


double stumpC(double z);
double stumpS(double z);

#endif // STUMPUFF_FUNCTIONS_H