/*
Trajectorize

KSP Kerbol System Bodies

This header file contains information for bodies in the Kerbol system.
*/

#ifndef KERBOL_SYSTEM_BODIES_H
#define KERBOL_SYSTEM_BODIES_H

#include "kerbol_system_types.h"

extern const Body kerbol_system_bodies[];

#endif // KERBOL_SYSTEM_BODIES_H